/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;

using Whisperstream.SerialComm.Framework.SMS;

namespace Whisperstream.SerialComm.Framework.Interfaces
{
	/// <summary>
	/// the delegate method used when new SMS messages are received when
	/// IncomingSMSEventsEnabled = true 
	/// </summary>
	public delegate void NewSMSHandler(object o, SMSMessage msg);

	/// <summary>
	/// The framework's SMS Message interface
	/// </summary>
	public interface ISMSModule
	{
		#region Events

		/// <summary>
		/// event fired when new incoming SMS messages are enabled
		/// </summary>
		event NewSMSHandler OnIncomingSMS;

		#endregion

		#region Methods

		/// <summary>
		/// get/set whether or not the event handler will be enabled
		/// </summary>
		bool IncomingSMSEventsEnabled{get;set;}

		/// <summary>
		/// determines whether or not incoming sms event messages should be
		/// written to the phone's inbox automatically or not
		/// </summary>
		bool WriteIncomingSMSEventsToPhone{get;set;}

		/// <summary>
		/// sends an sms  message
		/// </summary>
		/// <param name="msg">the message to send</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool SendMessage(PDUSubmit msg);

		/// <summary>
		/// Writes a message to the phone's memeory.  This method is called to write a message
		/// to the Sent/Unsent memory areas, any attempt to send a message to the Read/UnRead memory
		/// areas will cause an Arguments exception
		/// </summary>
		/// <param name="msg">the message to write to memory</param>
		/// <param name="wmo">the Memory storage area to write to</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool WriteMessageToMemory(PDUSubmit msg, WriteToMemoryOptions wmo);

		/// <summary>
		/// Writes a message to the phone's memeory.  This method is called to write a message
		/// to the Read/UnRead memory areas, any attempt to send a message to the Sent/Unsent memory
		/// areas will cause an Arguments exception
		/// </summary>
		/// <param name="msg">the message to write to memory</param>
		/// <param name="wmo">the Memory storage area to write to</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool WriteMessageToMemory(SMSMessage msg, WriteToMemoryOptions wmo);

		/// <summary>
		/// Send a previously stored message to the same recipient again
		/// </summary>
		/// <param name="msgNum">the message index in memory</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool SendStoredMessage(int msgNum);

		/// <summary>
		///  Send a previously stored message to a new recipient
		/// </summary>
		/// <param name="msgNum">the message index in memory</param>
		/// <param name="toNumber">the new recipient</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool SendStoredMessage(int msgNum, string toNumber);

		/// <summary>
		/// Deletes a message from the phone's memory
		/// </summary>
		/// <param name="msgNum">the message index</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool DeleteMessage(int msgNum);

		/// <summary>
		/// Selects the active memory store (usually SimCard or Phone Memory)
		/// </summary>
		/// <param name="ms">the memory store</param>
		/// <returns>true if the operation was successful, false otherwise</returns>
		bool SelectActiveMessageStore(MemoryStore ms);
		
		/// <summary>
		/// gets /sets the preferred memory storage area from inbox,out box and incoming messages
		/// throws CommandFailedException if command fails
		/// </summary>
		PreferredMemoryStorage PreferredMemoryStorage{get;set;}

		/// <summary>
		/// Reads a message located at msgNum index in memory
		/// </summary>
		/// <param name="msgNum">the message index</param>
		/// <returns>the sms message or null if no message is found</returns>
		SMSMessage ReadMessage(int msgNum);

		/// <summary>
		/// Provides a list of all messages in the active memory
		/// </summary>
		/// <returns>the list of all SMSMessage objects in the active memory</returns>
		ArrayList ReadAllMessages();

		#endregion	
	}
}
