/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Net;
using System.Text.RegularExpressions;
using System.Web.Mail;

using Whisperstream.SerialComm.Framework.SMS;
using Whisperstream.SMSProcessor;
using Whisperstream.SMSProcessor.Interfaces;
using Whisperstream.SMSProcessor.Utility;

namespace Whisperstream.MyIpProcessor
{
	/// <summary>
	/// Implements IMessageProcessor interface to provide a method for obtaining
	/// the machine's external IP address
	/// </summary>
	public class IpProcessor : IMessageProcessor
	{
		private ProcessorHelperData processorHelperData;
		private ILog log;

		/// <summary>
		/// IpProcessor constructor
		/// </summary>
		public IpProcessor()
		{
		}

		#region IMessageProcessor Members

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		public string Author
		{
			get
			{
				return "Whisperstream Research Group (www.whisperstream.com)";
			}
		}

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		public string Version
		{
			get
			{
				return "0.1";
			}
		}

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		public string MessageCode
		{
			get
			{
				return "MYIP:";
			}
		}

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		/// <param name="processorHelperData"></param>
		public void SetHelperData(ProcessorHelperData processorHelperData)
		{
			this.processorHelperData = processorHelperData;
			log = processorHelperData.GetLogger(this.GetType());
		}

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		public string MessageCodeSyntax
		{
			get
			{
				return MessageCode + " <password> [<email@address.com>]";
			}
		}

		/// <summary>
		/// See the IMessageProcessor interface
		/// </summary>
		/// <param name="smsMessage"></param>
		public void ProcessMessageCode(SMSMessage smsMessage)
		{
			Regex rgx = new Regex(@"\S+ (\S+)\s*(\S*)");

			Match m = rgx.Match(smsMessage.Message);

			// make sure the sender knows the master password
			if(!m.Success || !processorHelperData.IsMasterPasswordMatch(m.Result("$1")))
			{
				log.Error("Incorrect Master Password:" + m.Result("$1"));
				return;
			}

			log.Info("Accessing Web");

			// get the address that gives the IP from the application settings
			string ipUrl = processorHelperData.GetSetting("MYIP_ipUrl");

			// make a request to this website in order to get the external ip address
			HttpWebComm request = new HttpWebComm();
			string result = request.GetUrl(ipUrl);

			string messageText = "";

			// make sure we got a valid IP address
			if(StringUtil.IsValidIPAddress(result))
				messageText = String.Format("X-{0} {1}", MessageCode, result);
			else 
				messageText = String.Format("X-{0} ERROR", MessageCode);

			// if no email address was given as a second parameter then send an sms message
			// back to the phone that originated the message
			if(m.Result("$2") == "")
			{
				log.Info("Returning result via GSM network");
				processorHelperData.IMobilePhoneAPI.SMS.SendMessage(
					new PDUSubmit(smsMessage.SenderNumber, messageText));
			} 
			else 
			{
				log.Info("Returning result via email");
				// send the return message as an email to the address specified in the message
				SendReplyAsMail(messageText, m.Result("$2"), processorHelperData.SmtpServer);
			}
		}

		private void SendReplyAsMail(string reply, string emailAddress, string smtpServer)
		{
			if(!StringUtil.IsValidEmailAddress(emailAddress))
			{
				log.Error("Invalid Email Address: " + emailAddress);
				return;
			}

			MailMessage message = new MailMessage();

			message.To = emailAddress;
			message.Subject = reply;
			message.Body = reply;
			message.From = @"""SMS Proc"" <sms@whisperstream.com>";

			System.Web.Mail.SmtpMail.SmtpServer = smtpServer;

			System.Web.Mail.SmtpMail.Send(message);
		}

		#endregion
	}
}
