/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;

using Whisperstream.NokiaAPI.Phonebook;
using Whisperstream.NokiaAPI.SMS;
using Whisperstream.NokiaAPI.PhoneSettings;
using Whisperstream.SerialComm.Framework.Interfaces;
using Whisperstream.SerialComm.IO;

namespace Whisperstream.NokiaAPI {

	/// <summary>
	/// Implements the IMobilePhoneAPI interface for Nokia mobile phones, specifically the Nokia 6210
	/// </summary>
	public class NokiaApi : IMobilePhoneAPI
	{
		private ISerialIO iSerialIO;
		private SMSModule smsModule;
		private PhonebookModule phonebookModule;
		private PhoneSettingsModule phoneSettingsModule;

		/// <summary>
		/// NokiaApi constructor
		/// </summary>
		/// <param name="comPort">the COM port for the serial connection</param>
		/// <param name="baudRate">the baud rate</param>
		public NokiaApi(string comPort, uint baudRate)
		{
			Initialise(comPort, baudRate);
		}

		/// <summary>
		/// NokiaApi constructor
		/// </summary>
		/// <param name="comPort">the COM port for the serial connection</param>
		public NokiaApi(string comPort)
		{
			Initialise(comPort, 0);
		}

		#region IMobilePhoneAPI Members

		/// <summary>
		/// See the IMobilePhoneAPI interface
		/// </summary>
		public ISerialIO SerialIO{ get { return iSerialIO; }}

		/// <summary>
		/// See the IMobilePhoneAPI interface
		/// </summary>
		public ISMSModule SMS{ get { return GetSMSModule(); }}

		/// <summary>
		/// See the IMobilePhoneAPI interface
		/// </summary>
		public IPhonebookModule Phonebook{ get { return GetPhonebookModule(); }}

		/// <summary>
		/// See the IMobilePhoneAPI interface
		/// </summary>
		public IPhoneSettingsModule PhoneSettings { get { return GetPhoneSettingsModule(); }}

		#endregion

		private void Initialise(string comPort, uint baudRate)
		{
			if(baudRate == 0)
				iSerialIO = SerialIOService.Instance.GetSerialConnection(comPort);
			else
				iSerialIO = SerialIOService.Instance.GetSerialConnection(comPort, baudRate);

			iSerialIO.CommandPauseInterval = 100;

			iSerialIO.Open();

			// turn echo off
			string result = iSerialIO.SendCommand("ATE0\r\n");
		}

		/// <summary>
		/// If we haven't already got an instance of an SMSModule then get one now
		/// </summary>
		/// <returns>an ISMSModule interface</returns>
		private ISMSModule GetSMSModule()
		{
			if(smsModule == null)
				smsModule = new SMSModule(iSerialIO);

			return (ISMSModule)smsModule;
		}

		/// <summary>
		/// If we haven't already got an instance of a PhonebookModule then get one now
		/// </summary>
		/// <returns>an IPhonebookModule interface</returns>
		private IPhonebookModule GetPhonebookModule()
		{
				if(phonebookModule == null)
					phonebookModule = new PhonebookModule(iSerialIO);

			return (IPhonebookModule)phonebookModule;
		}

		/// <summary>
		/// If we haven't already got an instance of a PhoneSettingsModule then get one now
		/// </summary>
		/// <returns>an IPhoneSettingsModule interface</returns>
		private IPhoneSettingsModule GetPhoneSettingsModule()
		{
			if(phoneSettingsModule == null)
				phoneSettingsModule = new PhoneSettingsModule(iSerialIO);

			return (IPhoneSettingsModule)phoneSettingsModule;
		}


	}
}