/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Threading;

using Whisperstream.NokiaAPI.Common;
using Whisperstream.SerialComm.Framework;
using Whisperstream.SerialComm.Framework.Interfaces;
using Whisperstream.SerialComm.Framework.Phonebook;
using Whisperstream.SerialComm.IO;

namespace Whisperstream.NokiaAPI.PhoneSettings
{
	/// <summary>
	/// Nokia 6210 implementation of the IPhoneSettingsModule interface
	/// </summary>
	public class PhoneSettingsModule : ApiModuleBase, IPhoneSettingsModule
	{
		private NumberFormatInfo numberFormatInfo = new NumberFormatInfo();
		
		/// <summary>
		/// PhoneSettingsModule constructor
		/// </summary>
		/// <param name="iSerialIO">an ISerialIO interface</param>
		public PhoneSettingsModule(ISerialIO iSerialIO) : base(iSerialIO)
		{
			numberFormatInfo.NumberDecimalSeparator = ".";
		}

		private string GetStringValue(string command, string pattern, string resultReplace)
		{
			string result = SendCommand(command);

			bool commandOK = Util.CheckResponseOK(ref result);

			if(commandOK)
			{
				Regex rgx = new Regex(pattern);
				Match m = rgx.Match(result);
				if(m.Success)
				{
					return m.Result(resultReplace);
				}
			} 

			throw new CommandFailedException(command);
		}

		private double GetDoubleValue(string command, string pattern, string resultReplace)
		{
			string result = SendCommand(command);

			bool commandOK = Util.CheckResponseOK(ref result);

			if(commandOK)
			{
				Regex rgx = new Regex(pattern);
				Match m = rgx.Match(result);
				if(m.Success)
				{
					double finalResult = 0.0;
					if(double.TryParse(
						m.Result(resultReplace).Replace(",","."),
						NumberStyles.Number,
						(IFormatProvider)numberFormatInfo,
						out finalResult))
						return finalResult;
				}
			} 

			throw new CommandFailedException(command);
		}

		private bool GetBoolValue(string command)
		{
			string result = SendCommand(command);
			return Util.CheckResponseOK(ref result);
		}

		#region IPhoneSettings Members

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public bool RestoreStoredProfile()
		{
			return GetBoolValue("ATZ\r\n");
		}
		
		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public double GetSignalQuality()
		{
			return GetDoubleValue("AT+CSQ\r\n", @"(\d+,\d+)", "$1");
		}

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public string GetManufacturerIdentification()
		{
			return GetStringValue("AT+CGMI\r\n", @"([\w ]+)\r\n", "$1");
		}

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public string GetFirmwareVersion()
		{
			return GetStringValue("AT+CGMR\r\n", @"([\w\. ]+)\r\n", "$1");
		}

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public double GetBatteryCharge()
		{
			return GetDoubleValue("AT+CBC\r\n", @"(\d+,\d+)", "$1");
		}

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public string GetEquipmentSerialNumber()
		{
			return GetStringValue("AT+CGSN\r\n", @"(\d+)\r\n", "$1");
		}

		/// <summary>
		/// See IPhoneSettingsModule interface
		/// </summary>
		/// <returns></returns>
		public string GetModelIdentification()
		{
			return GetStringValue("AT+CGMM\r\n", @"([\w ]+)\r\n", "$1");
		}

		#endregion
	}
}
