/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;
using System.Text.RegularExpressions;
using System.Threading;

using Whisperstream.NokiaAPI.Common;
using Whisperstream.SerialComm.Framework;
using Whisperstream.SerialComm.Framework.Interfaces;
using Whisperstream.SerialComm.Framework.Phonebook;
using Whisperstream.SerialComm.IO;

namespace Whisperstream.NokiaAPI.Phonebook
{
	/// <summary>
	/// Nokia 6210 implementation of the IPhonebookModule interface
	/// </summary>
	public class PhonebookModule : ApiModuleBase, IPhonebookModule
	{
		private Hashtable memoryStoreCache;

		/// <summary>
		/// PhonebookModule constructor
		/// </summary>
		/// <param name="iSerialIO">an ISerialIO interface</param>
		public PhonebookModule(ISerialIO iSerialIO) : base(iSerialIO)
		{
		}

		#region IPhonebookModule Members

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		public MemoryStore CurrentPhonebookMemoryStore
		{
			get { return GetCurrentPhonebookMemoryStore(); }
			set { SetCurrentPhonebookMemoryStore(value); }
		}
		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <param name="p"></param>
		/// <returns></returns>
		public bool AddPhonebookEntry(PhonebookEntry p)
		{
			string result = SendCommand(
				"AT+CPBW="+ ",\"" + p.Number + "\"," + (int)p.NumberType + ",\"" + p.Name + "\"\r\n");
			
			return Util.CheckResponseOK(ref result);
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <param name="p"></param>
		/// <returns></returns>
		public bool EditPhonebookEntry(PhonebookEntry p)
		{
			string result = SendCommand(
				"AT+CPBW=" + p.Index + ",\"" + p.Number + "\"," + (int)p.NumberType + ",\"" + p.Name + "\"\r\n");
			
			return Util.CheckResponseOK(ref result);
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <param name="index"></param>
		/// <returns></returns>
		public bool RemovePhonebookEntry(int index)
		{
			string result = SendCommand("AT+CPBW=" + index + "\r\n");

			return Util.CheckResponseOK(ref result);
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <param name="entry"></param>
		/// <returns></returns>
		public PhonebookEntry ReadPhonebookEntry(int entry)
		{
			PhonebookEntry phonebookEntry = null;
			// get the phonebook entry at index "entry"
			string result = SendCommand("AT+CPBR=" + entry +"\r\n");

			if(Util.CheckResponseOK(ref result))
			{
				// matches : +CPBR: 1,"55555555",129,"Some Entry"
				Regex rgx = new Regex(@"\+CPBR: (\d+),""([\+\d]+)"",(\d+),""([\w\s]+)""");
		
				Match match = rgx.Match(result);

				if(match.Success)
				{
					phonebookEntry = new PhonebookEntry(
						int.Parse(match.Result("$1")),					// index
						match.Result("$2"),								// number
						(NumberType)int.Parse(match.Result("$3")),		// number type
						match.Result("$4"));							// name
				}
			}

			return phonebookEntry;
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <param name="ms"></param>
		/// <returns></returns>
		public bool IsMemoryStoreSupported(MemoryStore ms)
		{
			if(memoryStoreCache == null)
				GetSupportedMemoryStores();

			return memoryStoreCache.Contains(ms);
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <returns></returns>
		public ArrayList GetSupportedMemoryStores()
		{
			memoryStoreCache = new Hashtable();

			string command = "AT+CPBS=?\r\n";
			string result = SendCommand(command);

			if(Util.CheckResponseOK(ref result))
			{
				// matches: +CPBS: ("ME","DC","MC","RC","ON","SM","FD")
				Regex rgx = new Regex(@"\""(\w+)\""");

				MatchCollection mc = rgx.Matches(result);

				foreach(Match m in mc)
				{
					memoryStoreCache.Add(FrameworkUtil.StringToMemoryStore(m.Result("$1")), null);
				}
			}

			return new ArrayList(memoryStoreCache.Keys);
		}

		/// <summary>
		/// See IPhonebookModule interface
		/// </summary>
		/// <returns></returns>
		public Hashtable GetAddressBook()
		{
			Hashtable phonebook = new Hashtable();
			string command = null;
			Regex rgx;

			// get the maximum amount of phonebook entries allowable
			string result = SendCommand("AT+CPBR=?\r\n");

			if(Util.CheckResponseOK(ref result))
			{
				rgx = new Regex(@"\+CPBR: \((\d+)-(\d+)\)");
		
				Match m = rgx.Match(result);

				if(m.Success)
					command = "AT+CPBR=" + m.Result("$1") + "," +  m.Result("$2") + "\r\n";

			} 

			if(command == null)
				// use a default number
				command = "AT+CPBR=1,200\r\n";

			
			// get all the phonebook entries from the phone
			result = SendCommand(command);

			if(Util.CheckResponseOK(ref result))
			{
				// matches : +CPBR: 1,"55555555",129,"Some Entry"
				rgx = new Regex(@"\+CPBR: (\d+),""([\+\d]+)"",(\d+),""([\w\s]+)""");
		
				foreach(Match match in rgx.Matches(result))
				{
					PhonebookEntry entry = new PhonebookEntry(
						int.Parse(match.Result("$1")),					// index
						match.Result("$2"),								// number
						(NumberType)int.Parse(match.Result("$3")),		// number type
						match.Result("$4"));							// name
			
					// add the phone entries to the cache using the phone number as a key
					phonebook.Add(entry.Number, entry);
				}
			}
			
			return phonebook;
		}

		#endregion

		private MemoryStore GetCurrentPhonebookMemoryStore()
		{
			string command = "AT+CPBS?\r\n";
			string result = SendCommand(command);

			// macthes: +CPBS: "SM",4,200
			Regex rgx = new Regex(@"\+CPBS: \""(\w+)\""[\d,]+");

			if(Util.CheckResponseOK(ref result))
			{
				Match m = rgx.Match(result);

				if(m.Success)
					return FrameworkUtil.StringToMemoryStore(m.Result("$1"));
			}

			throw new CommandFailedException(command, result);
		}

		private void SetCurrentPhonebookMemoryStore(MemoryStore ms)
		{
			string command = @"AT+CPBS=\""" + FrameworkUtil.MemoryStoreToString(ms) + @"\""\r\n";
			string result = SendCommand(command);

			if(!Util.CheckResponseOK(ref result))
			{
				throw new CommandFailedException(command, result);
			}
		}

	}
}
