/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;
using System.Text.RegularExpressions;
using System.Threading;

using Whisperstream.NokiaAPI.Common;
using Whisperstream.NokiaAPI.Phonebook;
using Whisperstream.SerialComm.Framework;
using Whisperstream.SerialComm.Framework.Interfaces;
using Whisperstream.SerialComm.Framework.SMS;
using Whisperstream.SerialComm.Framework.Phonebook;
using Whisperstream.SerialComm.IO;

namespace Whisperstream.NokiaAPI.SMS
{
	/// <summary>
	/// Nokia 6210 implementation of the ISMSModule interface
	/// </summary>
	public class SMSModule : ApiModuleBase, ISMSModule
	{
		private ArrayList messageCache;
		private Regex incomingSMSRegex;
		private ReceivedDataHandler receivedDataHandler;
		private bool incomingSMSEventsEnabled = false;
		private Hashtable phonebook;
		private bool writeIncomingSMSEventsToPhone = true;
		private DateTime lastPhonebookRefresh = DateTime.MinValue;

		/// <summary>
		/// SMSModule constructor
		/// </summary>
		/// <param name="iSerialIO">an ISerialIO interface</param>
		public SMSModule(ISerialIO iSerialIO) : base(iSerialIO)
		{
			incomingSMSRegex = new Regex(@"\+CMT: [\d+,]+\r\n([0-9,A-F]+)", RegexOptions.Compiled);

			receivedDataHandler = new ReceivedDataHandler(OnSerialReceive);
		}

		#region ISMSModule Members

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		public event NewSMSHandler OnIncomingSMS;

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		public PreferredMemoryStorage PreferredMemoryStorage
		{
			get{ return GetPreferredMemoryStorage(); }
			set{ SetPreferredMemoryStorage(value); }
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		public bool WriteIncomingSMSEventsToPhone 
		{ 
			get { return writeIncomingSMSEventsToPhone; } 
			set { writeIncomingSMSEventsToPhone = value; }
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		public bool IncomingSMSEventsEnabled 
		{
			get
			{
				return incomingSMSEventsEnabled;
			}

			set
			{
				if(value != incomingSMSEventsEnabled)
				{
					string command;

					if(value)
						// sends incoming messages to the serial interface
						command = "AT+CNMI=1,2\r\n";
					else
						// sends incoming messages directly to memory storage
						command = "AT+CNMI=0,0\r\n";

					string result = SendCommand(command);

					// make sure the command executed ok
					if(Util.CheckResponseOK(ref result))
					{
						incomingSMSEventsEnabled = value;
						if(incomingSMSEventsEnabled)
							// subscribe to the serial port's onreceive event
							iSerialIO.OnReceive += receivedDataHandler;
						else
							// unsubscribe from the serial port's onreceive event
							iSerialIO.OnReceive -= receivedDataHandler;
					}
					else 
					{
						// the command returned an unexpected result, throw an
						// exception because otherwise the user won't know why
						// if they enabled/disabled incoming events, the status is still
						// the same
						throw new CommandFailedException(command);
					}
				}
			}
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msg"></param>
		/// <returns></returns>
		public bool SendMessage(PDUSubmit msg)
		{
			string pduString = SMSConvert.EncodeTextMessageData(msg);
			int pduLen = pduString.Length/2 - 1;

			return StoreOrSendMessage(
				"AT+CMGS=" + pduLen + "\r\n",
				pduString);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msg"></param>
		/// <param name="wmo"></param>
		/// <returns></returns>
		public bool WriteMessageToMemory(PDUSubmit msg, WriteToMemoryOptions wmo)
		{
			// make sure the message is written to the correct memory area
			if(wmo == WriteToMemoryOptions.ReceivedRead || wmo == WriteToMemoryOptions.ReceivedUnread) 
				throw new ArgumentException("Incorrect memory store", "WriteToMemoryOptions wmo");

			string pduString = SMSConvert.EncodeTextMessageData(msg);

			return WriteMessageToMemory(pduString, wmo);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msg"></param>
		/// <param name="wmo"></param>
		/// <returns></returns>
		public bool WriteMessageToMemory(SMSMessage msg, WriteToMemoryOptions wmo)
		{
			// make sure the message is written to the correct memory area
			if(wmo == WriteToMemoryOptions.StoredUnsent || wmo == WriteToMemoryOptions.StoredSent) 
				throw new ArgumentException("Incorrect memory store", "WriteToMemoryOptions wmo");

			// make sure that the pdu data reflects the rest of the data contained
			// in the SMSMessage structure
			if(!msg.IsValid)
				msg = SMSConvert.ReEncodeSMSMessage(msg);

			return WriteMessageToMemory(msg.RawPDUData, wmo);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="rawPDUData"></param>
		/// <param name="wmo"></param>
		/// <returns></returns>
		private bool WriteMessageToMemory(string rawPDUData, WriteToMemoryOptions wmo)
		{
			int tpdLength = SMSUtil.CalculateTPDLength(rawPDUData);

			return StoreOrSendMessage(
				"AT+CMGW=" + tpdLength + "," + ((int)wmo).ToString() + "\r\n",
				rawPDUData);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msgNum"></param>
		/// <returns></returns>
		public bool SendStoredMessage(int msgNum)
		{
			string result = SendCommand("AT+CMSS=" + msgNum.ToString() + "\r\n");

			return Util.CheckResponseOK(ref result);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msgNum"></param>
		/// <param name="toNumber"></param>
		/// <returns></returns>
		public bool SendStoredMessage(int msgNum, string toNumber)
		{
			string numberType = (toNumber.StartsWith("+") ? "145" : "129");

			string result = SendCommand(
				"AT+CMSS=" + msgNum.ToString() + 
				",\"" + toNumber + "\"," + numberType + "\r\n");

			return false;
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msgNum"></param>
		/// <returns></returns>
		public SMSMessage ReadMessage(int msgNum)
		{
			string command = "AT+CMGR=" + msgNum + "\r\n";
			string result = SendCommand(command);

			bool commandOK = Util.CheckResponseOK(ref result);

			if(!commandOK)
				throw new CommandFailedException(command);

			Regex rgx = new Regex(@"\+CMGR: (\d+),,(\d+)\r\n([A-F,0-9]+)");

			Match m = rgx.Match(result);

			if(m.Success)
			{
				PDUDeliver smsDecode;

				// catch any exceptions that might be thrown because a message
				// might not be a normal message. i.e. a status report
				try
				{
					smsDecode = new PDUDeliver(
						m.Result("$4"),			    // pdudata
						-1,							// message memory index
						int.Parse(m.Result("$1"))); // message index

					SMSMessage smsMessage = SMSConvert.DecodePDUData(smsDecode);

					// check the phonebook for an entry matching the sender's number,
					// so that we can fill in the sender's name field
					smsMessage.SenderName = CheckPhonebookEntries(smsMessage.SenderNumber);

					return smsMessage;
				}
				catch(ArgumentException)
				{
				}
			}

			throw new CommandFailedException("Unable to match data: " + result);
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <returns></returns>
		public ArrayList ReadAllMessages()
		{
			ArrayList messageList = new ArrayList();

			// get the currenct memory storage options and clone a copy of them
			PreferredMemoryStorage orig = this.PreferredMemoryStorage;
			PreferredMemoryStorage temp = orig.Clone();

			// get all messages stored on the sim card
			temp.Inbox = SMSMemoryStore.SimCard;
			this.PreferredMemoryStorage = temp;
			messageList = this.ReadMessagesFromMemory(SMSMemoryStore.SimCard);

			// get all messages stored on the phone's internal memory
			temp.Inbox = SMSMemoryStore.OnPhoneMemory;
			this.PreferredMemoryStorage = temp;
			messageList.AddRange(this.ReadMessagesFromMemory(SMSMemoryStore.OnPhoneMemory));

			// reset the memory storage options to their original values
			this.PreferredMemoryStorage = orig;

			return messageList;
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="msgNum"></param>
		/// <returns></returns>
		public bool DeleteMessage(int msgNum)
		{
			string result = SendCommand("AT+CMGD=" + msgNum + "\r\n");

			bool commandOK = Util.CheckResponseOK(ref result);

			if(commandOK)
			{
				messageCache = null;
			}

			return commandOK;
		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <param name="ms"></param>
		/// <returns></returns>
		public bool SelectActiveMessageStore(MemoryStore ms)
		{
			string result = SendCommand("AT+CPMS=" + "\"" + FrameworkUtil.MemoryStoreToString(ms) + "\"" + "\r\n");

			return Util.CheckResponseOK(ref result);
		}

		#endregion

		/// <summary>
		/// The handler for generating OnIncomingSMS Messages.  Responsible for
		/// scanning all serialIO data and extracting incoming SMS Messages
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OnSerialReceive(object sender, ReceivedDataEventArgs e)
		{
			// match incoming sms messages
			foreach(Match m in incomingSMSRegex.Matches(e.ReceivedData))
			{
				PDUDeliver smsDecode = new PDUDeliver(
					m.Result("$1"),
					Constant.UNKNOWN_MESSAGEMEMORYINDEX,
					Constant.UNKNOWN_MESSAGEINDEX);

				// convert the pdu encoded message into a fully translated SMSMessage object
				SMSMessage message = SMSConvert.DecodePDUData(smsDecode);

				if(writeIncomingSMSEventsToPhone)
				{
					// write the incoming sms to stored memory so that it will not be lost
					WriteMessageToMemory(message, WriteToMemoryOptions.ReceivedUnread);
				}

				if(OnIncomingSMS != null)
					OnIncomingSMS(this, message);
			}			
		}

		private bool StoreOrSendMessage(string cmd, string pdu)
		{
			string result = SendCommand("AT+CMGF=0\r\n");

			bool commandOK = Util.CheckResponseOK(ref result);

			if(commandOK)
			{
				result = SendCommand(cmd, new string[] { ">", "ERROR\r\n" });
				if(result.EndsWith(">"))
				{
					result = SendCommand(pdu + CTRL.Z);
					commandOK = Util.CheckResponseOK(ref result);
				}
			}

			return commandOK;
		}

		/// <summary>
		/// returns the name matching the phone number if it exists in the phone's phonebook
		/// </summary>
		/// <param name="number">the number to check</param>
		/// <returns>the name of the number's owner or null</returns>
		private string CheckPhonebookEntries(string number)
		{
			// refresh the phonebook entries every 5 minutes
			if(lastPhonebookRefresh < DateTime.Now.Subtract(new TimeSpan(0,0,5,0,0)))
			{
				phonebook = new PhonebookModule(iSerialIO).GetAddressBook();
				lastPhonebookRefresh = DateTime.Now;
			}

			return (phonebook.Contains(number) ? ((PhonebookEntry)phonebook[number]).Name : null);
		}

		private PreferredMemoryStorage GetPreferredMemoryStorage()
		{
			string command = "AT+CPMS?\r\n";
			string result = SendCommand(command);

			if(Util.CheckResponseOK(ref result))
			{
				// matches: +CPMS: "SM",3,30,"SM",3,30,"SM",3,30
				Regex rgx = new Regex(@"\+CPMS: \""(\w+)\""[\d,]+\""(\w+)\""[\d,]+\""(\w+)\""[\d,]+");

				Match m = rgx.Match(result);

				if(m.Success)
				{
					PreferredMemoryStorage preferredMemoryStorage = new PreferredMemoryStorage();
					preferredMemoryStorage.Inbox = FrameworkUtil.StringToSMSMemoryStore(m.Result("$1"));
					preferredMemoryStorage.Outbox = FrameworkUtil.StringToSMSMemoryStore(m.Result("$2"));
					preferredMemoryStorage.Incoming = FrameworkUtil.StringToSMSMemoryStore(m.Result("$3"));

					return preferredMemoryStorage;
				}
			}

			throw new CommandFailedException(command, result);
		}

		private void SetPreferredMemoryStorage(PreferredMemoryStorage preferredMemoryStorage)
		{
			string command = String.Format(@"AT+CPMS=""{0}"",""{1}"",""{2}""",
				FrameworkUtil.SMSMemoryStoreToString(preferredMemoryStorage.Inbox),
				FrameworkUtil.SMSMemoryStoreToString(preferredMemoryStorage.Outbox),
				FrameworkUtil.SMSMemoryStoreToString(preferredMemoryStorage.Incoming)) + "\r\n";

			string result = SendCommand(command);

			if(!Util.CheckResponseOK(ref result))
				throw new CommandFailedException(command, result);

		}

		/// <summary>
		/// See ISMSModule interface
		/// </summary>
		/// <returns></returns>
		private ArrayList ReadMessagesFromMemory(SMSMemoryStore memoryStore)
		{
			ArrayList messageList = new ArrayList();

			string result = SendCommand("AT+CMGL=4\r\n");

			bool commandOK = Util.CheckResponseOK(ref result);

			if(!commandOK)
				return messageList;

			Regex rgx = new Regex(@"\+CMGL: (\d+),(\d+),,(\d+)\r\n([A-F,0-9]+)");

			MatchCollection mc = rgx.Matches(result);

			if(mc.Count > 0)
			{
				foreach(Match m in mc)
				{
					PDUDeliver smsDecode;

					// catch any exceptions that might be thrown because a message
					// might not be a normal message. i.e. a status report
					try
					{
						smsDecode = new PDUDeliver(
							m.Result("$4"),			   // pdudata
							int.Parse(m.Result("$2")), // message memory index
							int.Parse(m.Result("$1"))); // message index
					}
					catch(ArgumentException)
					{
						continue;
					}

					SMSMessage smsMessage = SMSConvert.DecodePDUData(smsDecode);

					// check the phonebook for an entry matching the sender's number,
					// so that we can fill in the sender's name field
					smsMessage.SenderName = CheckPhonebookEntries(smsMessage.SenderNumber);

					smsMessage.MessageMemoryStore = memoryStore;

					messageList.Add(smsMessage);

					Thread.Sleep(0);
				}
			}

			return messageList;
		}
	}
}
