/*
 * Non-Commercial License
 * 
 * Owner: Whisperstream Research Group, Copyright 2004
 * 
 * DEFINITIONS
 * "Software" means source code (libraries) and associated documentation and data.
 * "Research Use" means research, evaluation, and educational use, excluding use or
 *  distribution for direct or indirect commercial (including strategic) gain or advantage.
 * 
 * PURPOSE
 * The purpose of licensing the Software is to promote education and research and to allow
 * feedback from the development community.
 * 
 * TERMS
 * You are granted a perpetual, royalty-free, non-exclusive license, subject to the
 * claims of any third parties, to use the Software for Research Use only, provided you
 * agree to and comply with any and all conditions in this license. COMMERCIAL USE AND 
 * DISTRIBUTION OF THE SOFTWARE IS NOT PERMITTED UNDER THIS LICENSE. 
 * If you are interested in other licensing alternatives, please contact the copyright owners.
 * 
 * Contributors grant to the copyright owners a world-wide, royalty-free, perpetual, 
 * non-exclusive license, subject to third party intellectual property claims: to use,
 * reproduce, modify, display, perform, sub-license, sell, and distribute the contributor's
 * modifications in software developed by the copyright owners, even if such software is released
 * under a different license. 
 * 
 * BECAUSE THE SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE SOFTWARE,
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 * HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE 
 * PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION. 

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER,
 * OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE SOFTWARE AS PERMITTED ABOVE, BE LIABLE
 * TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
 * RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE
 * WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES. 
*/
using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Threading;

using Whisperstream.SMSProcessor.Interfaces;
using Whisperstream.SerialComm.Framework;
using Whisperstream.SerialComm.Framework.Interfaces;
using Whisperstream.SerialComm.Framework.SMS;

namespace Whisperstream.SMSProcessor
{
	/// <summary>
	/// Processes incoming SMS messages by distributing them to the dynamically loaded assemblies of 
	/// IMessageProcessor implementations
	/// </summary>
	public class MessageProcessor
	{
		private ProcessorHelperData processorHelperData;
		private Hashtable codeMatchTable = new Hashtable();
		private ILog log;

		/// <summary>
		/// MessageProcessor constructor
		/// </summary>
		/// <param name="processorHelperData">Helper data that may be useful for IMessageProcessor implementations</param>
		/// <param name="processorLibDir">The directory where assemblies of IMessageProcessor implementations are located</param>
		public MessageProcessor(ProcessorHelperData processorHelperData, string processorLibDir)
		{
			// make sure processorHelperData is not null
			if(processorHelperData == null)
				throw new ArgumentException("processorHelperData == null", "processorHelperData");
			else
				this.processorHelperData = processorHelperData;

			log = processorHelperData.GetLogger(this.GetType());

			// make sure we have a location to search for IMessageProcessor implementations
			if(processorLibDir == null || processorLibDir == "")
				throw new ArgumentException("bad param", "processorLibDir");
			else
				// dynamically load the IMessageProcessor implementation assemblies
				LoadUserAssemblies(processorLibDir);

			// subscribe and enable the on incoming sms events mechanism
			processorHelperData.IMobilePhoneAPI.SMS.OnIncomingSMS += new NewSMSHandler(OnIncomingSMS);
			processorHelperData.IMobilePhoneAPI.SMS.IncomingSMSEventsEnabled = true;
		}

		private void LoadAssembly(string assemblyPath)
		{
			Assembly assembly = Assembly.LoadFrom(assemblyPath);
   
			// Walk through each type in the assembly
			foreach (Type type in assembly.GetTypes ()) 
			{
				// Pick up a class
				if (type.IsClass == true) 
				{
					// If it does not implement the IMessageProcessor Interface, skip it
					if (type.GetInterface ("Whisperstream.SMSProcessor.Interfaces.IMessageProcessor") != null) 
					{
						// If however, it does implement the IBase Interface,
						// create an instance of the object
						IMessageProcessor iMessageProcessor = (IMessageProcessor)Activator.CreateInstance(type);

						// set the implmentation's ProcessorHelperData member variable
						iMessageProcessor.SetHelperData(processorHelperData);

						// add the IMessageProcessor instances to the Code Match Table if there isn't
						// already and existing signature
						string messageCode = iMessageProcessor.MessageCode.ToUpper();
						SMSEventExecutor smsEventExceutor;
						if(!codeMatchTable.ContainsKey(messageCode))
							codeMatchTable.Add(messageCode, new SMSEventExecutor());

						smsEventExceutor = (SMSEventExecutor)codeMatchTable[messageCode];
						smsEventExceutor.AddMessageProcessor(iMessageProcessor);

						log.Info("Added New Message Processor");
						log.Info("\tAuthor            : " + iMessageProcessor.Author);
						log.Info("\tVersion           : " + iMessageProcessor.Version);
						log.Info("\tMessageCode       : " + iMessageProcessor.MessageCode);
						log.Info("\tMessageCodeSyntax : " + iMessageProcessor.MessageCodeSyntax);
					}
				}
			}
		}

		private void LoadUserAssemblies(string processorLibPath)
		{
			DirectoryInfo d = new DirectoryInfo(processorLibPath);
			foreach (FileInfo file in d.GetFiles("*.dll"))
			{   
				//LoadAssembly(file.Name.Replace(file.Extension, ""));
				LoadAssembly(file.FullName);
			}
		}


		private void OnIncomingSMS(object o, SMSMessage msg)
		{
			Regex rgx = new Regex(@"\S+");

			// extract the code
			Match m = rgx.Match(msg.Message);

			// log the message
			log.Message(msg);

			if(m.Success)
			{
				// convert the code to upper case
				string code = m.Result("$0").ToUpper();

				// check the Code Match Table for a matching MessageCode
				if(codeMatchTable.Contains(code))
				{
					log.Info("Processing Message with code: " + code);
					SMSEventExecutor smsEventExecutor = (SMSEventExecutor)codeMatchTable[code];
					try
					{
						// get the sms executor to send the message to any listeners
						smsEventExecutor.SendMessageEvent(msg);
					} 
					catch(Exception e)
					{
						log.Error("Exception occurred for Code: " + code, e);
					}
				}
			} 
			else if(!processorHelperData.IMobilePhoneAPI.SMS.WriteIncomingSMSEventsToPhone)
			{
				// if incoming sms message events are not already being written to memory, then only
				// write the ones that don't actually match any code back to memory
				processorHelperData.IMobilePhoneAPI.SMS.WriteMessageToMemory(msg, WriteToMemoryOptions.ReceivedUnread);
			}
		}

		/// <summary>
		/// Used to send incoming sms message events to all IMessageProcessor implementations
		/// currently loaded
		/// </summary>
		private class SMSEventExecutor
		{
			private delegate void EventHandler(SMSMessage smsMessage);
			private ArrayList messageProcessors = new ArrayList();
			private event EventHandler OnMessage;

			/// <summary>
			/// Sends the incoming smsmessage to any listeners subscribed to the
			/// OnMessage event
			/// </summary>
			/// <param name="smsMessage">the incoming sms message</param>
			public void SendMessageEvent(SMSMessage smsMessage)
			{		
				if(OnMessage != null)
				{
					// send all the messages out to the listeners asynchronously
					foreach(EventHandler eventHandler in OnMessage.GetInvocationList())
					{
						eventHandler(smsMessage);
						Thread.Sleep(0);
					}
				}
			}

			/// <summary>
			/// Adds the a IMessageProcessor implementation to the list of subscribers of
			/// the OnMessage event
			/// </summary>
			/// <param name="iMessageProcessor">the IMessageProcessor implementation</param>
			public void AddMessageProcessor(IMessageProcessor iMessageProcessor)
			{
				messageProcessors.Add(iMessageProcessor);
				OnMessage += new EventHandler(iMessageProcessor.ProcessMessageCode);
			}

			/// <summary>
			/// Gets the list of IMessageProcessor implementations current subscribing to the
			/// OnMessage event
			/// </summary>
			/// <returns>An arraylist of IMessageProcessor implementations</returns>
			public ArrayList GetProcessorsList()
			{
				return (ArrayList)messageProcessors.Clone();
			}
		}
	}
}
